/* abcview.c */

/*

ABC Viewer, a program to display abc format music files on the Palm Pilot.

    Copyright (C) 1998,1999  Dave Holland  dave@zenda.demon.co.uk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <Pilot.h>
#include "callback.h"

#include "abcview.h"
#include "parse.h"

#include "abcviewRsc.h"


/* global variables */

DmOpenRef	MemoPadDB;
Word			ABCcategory;

/* the mystical "a" pointer */
CharPtr		a;

UInt			defaultLength;

UInt	pplet,qplet,rplet;

/* the main symbol array */
struct SYMBOL	ABCsymbols[maxSymbols];
UInt			nsyms=0;

/* the text array */
char			ABCtext[maxText][maxTextLen];
Int			ntext=0;

UInt			CurrentRecord=noRecordSelected;
Int			TopVisibleRecord=0;
Word			MemosInCategory;
UInt			TopStave=0;
Word			lastRecordNum;


/* show an alert box */
void Alert(CharPtr p1, CharPtr p2, CharPtr p3)
{
	FrmCustomAlert(DebugAlert,p1,p2,p3);
}

void AlertI(CharPtr p1, CharPtr p2, int i)
{
	char num[20];

	StrIToA(num,i);
	FrmCustomAlert(DebugAlert,p1,p2,num);
}


/* draw record in main view's table */
void MainViewDrawRecord(VoidPtr table, Word row, Word column,
	RectanglePtr bounds)
{
	VoidHand	recHandle;
	CharPtr recText;
	UInt	recordNum=0;
	FontID	currFont;
	Int		textLen,width;
	Boolean	fits;
	char	txt[100];
	char	num[10];
	UInt	pos;

	CALLBACK_PROLOGUE

	/* Get the record number matching the table item to draw.
	 * The record number is stored in the intValue field of the item. */
	recordNum=TblGetItemInt(table,row,0);

	/* Lock down the record from the database */
	recHandle=DmGetRecord(MemoPadDB,recordNum);
	recText=MemHandleLock(recHandle);

	/* draw it! */

	currFont=FntSetFont(stdFont);

	width=bounds->extent.x - 2;

	pos=DmPositionInCategory(MemoPadDB,recordNum,ABCcategory)+1;

	StrCopy(txt,"");
	/* TODO: fix alignment */
	if(pos<10)
		StrCat(txt,"  ");
	StrIToA(num,pos);
	StrCat(txt,num);
	StrCat(txt,".  ");
	if(pos<10)
		StrCat(txt," "); /* necessary? */

	/* cleverness with "X:" */
	if(StrNCompare(recText,"X:",2)==0)
	{
		while((*recText != '\n') && (*recText != '\0'))
			recText++;
		if(*recText == '\n')
			recText++;
	}

	/* cleverness with "T:" */
	if(StrNCompare(recText,"T:",2)==0)
	{
		recText+=2;
	}

	StrNCat(txt,recText,50);
	txt[50]='\0'; /* TODO: fix this! */

	textLen=StrLen(txt);
	FntCharsInWidth(txt,&width,&textLen,&fits);

	WinDrawChars(txt,textLen,bounds->topLeft.x,bounds->topLeft.y);

	FntSetFont(currFont);

	/* unlock and release */
	MemHandleUnlock(recHandle);
	DmReleaseRecord(MemoPadDB,recordNum,false); /* not dirty */

	CALLBACK_EPILOGUE
}

/* load database records into main table */
void MainViewLoadTable(FormPtr frm,UInt recordNum)
{
	Word		row,numRows;
	TablePtr	table;
	VoidHand	record;
	ScrollBarPtr bar;
	UInt		howmany=0;
	int			max;

	table=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainTable));

	/* iterate over each row in the table */
	numRows=TblGetNumberOfRows(table);
	for(row=0; row<numRows; row++, recordNum++)
	{
		record=DmQueryNextInCategory(MemoPadDB,&recordNum,ABCcategory);
		if(record)
		{
			/* we want to draw this one */
			TblSetItemStyle(table,row,0,customTableItem);
			TblSetItemInt(table,row,0,recordNum);
			TblSetRowUsable(table,row,true);
			lastRecordNum=recordNum;
		}
		else
		{
			/* more table rows than records -- set unusable */
			TblSetRowUsable(table,row,false);
		}

		/* mark invalid => force redraw */
		TblMarkRowInvalid(table,row);
	}

	howmany=DmNumRecordsInCategory(MemoPadDB,ABCcategory);

	if(howmany>numRows)
		max=howmany-numRows;
	else
		max=0;

	//AlertI("max","=",max);

	/* set scroll bar */
	bar=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainScroll));
	SclSetScrollBar(bar,
		DmPositionInCategory(MemoPadDB,TopVisibleRecord,ABCcategory),
		0,max, /* min, max */
		numRows); /* page size */
}


/* load the memopad records into the table */
void MainViewLoadRecords(FormPtr frm)
{
	TablePtr	table;
	Word		recordNum;
	Word		rowsInTable;

	table=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainTable));
	rowsInTable=TblGetNumberOfRows(table);

	/* if the current record is valid, try to get it on-screen */
	if (CurrentRecord != noRecordSelected)
	{
		/* if current record before top, move top to current */
		if (CurrentRecord < TopVisibleRecord)
		{
			TopVisibleRecord=CurrentRecord;
		}
		else
		{
			/* if current record off the bottom, make bottom the current */
			recordNum=TopVisibleRecord;
			DmSeekRecordInCategory(MemoPadDB,&recordNum,rowsInTable-1,
				dmSeekForward,ABCcategory);
			if(recordNum<CurrentRecord)
				TopVisibleRecord=CurrentRecord;
		}
	}

	/* try to show a full screen of records, when working near the
	 * end of the list */
	recordNum=dmMaxRecordIndex;
	DmSeekRecordInCategory(MemoPadDB, &recordNum, rowsInTable-1,
		dmSeekBackward, ABCcategory);
	TopVisibleRecord=min(TopVisibleRecord,recordNum);

	MainViewLoadTable(frm,TopVisibleRecord);

	/* set callback */
	TblSetCustomDrawProcedure(table,0,MainViewDrawRecord);

	TblSetColumnUsable(table,0,true);

}


void draw_nat(UInt x, UInt y)
{
	WinDrawLine(x,y-3, x,y+1);
	WinDrawLine(x,y-1, x+2,y-1);
	WinDrawLine(x,y+1, x+2,y+1);
	WinDrawLine(x+2,y-1, x+2,y+3);
}

void draw_sharp(UInt x, UInt y)
{
	WinDrawLine(x+1,y-3, x+1,y+3);
	WinDrawLine(x+3,y-3, x+3,y+3);
	WinDrawLine(x,y-1, x+4,y-1);
	WinDrawLine(x,y+1, x+4,y+1);
	/* WinEraseLine(x+2,y, x+2,y); */
}

void draw_flat(UInt x, UInt y)
{
	WinDrawLine(x,y-5, x,y+1);
	WinDrawLine(x,y+1, x+2,y-1);
	WinDrawLine(x+2,y-1, x,y-1);
}

/* draw a particular symbol */
void drawsymbol(UInt x, UInt y, struct SYMBOL s)
{
	RectangleType r;
	UInt sy;
	char tt[20];
	int i;

	sy=y+(STAVE_SEP/2)-8;

	switch(s.type)
	{
		case NOTE:

			/* accidentals */
			switch(s.acc)
			{
				case A_SH:
					draw_sharp(x+1,sy-2*(s.pitch-26));
					x+=6;
					break;

				case A_NT:
					draw_nat(x+1,sy-2*(s.pitch-26));
					x+=4;
					break;

				case A_FL:
					draw_flat(x+1,sy-2*(s.pitch-26));
					x+=4;
					break;

				case A_DS:
					break;

				case A_DF:
					break;
			}

			/* put the ledger line(s) in... */
			if(s.pitch<17)
			{
				for(i=16;i>=s.pitch;i-=2)
				{
					WinDrawLine(x,sy-2*(i-26), x+4,sy-2*(i-26));
				}
			}
			if(s.pitch>27)
			{
				for(i=28;i<=s.pitch;i+=2)
				{
					WinDrawLine(x,sy-2*(i-26), x+4,sy-2*(i-26));
				}
			}

			/* then the body of the note can overwrite if necessary */
			r.topLeft.x=x+1;
			r.topLeft.y=sy-2*(s.pitch-26)-1;
			r.extent.x=3;
			r.extent.y=3;
			WinDrawRectangle(&r,0);
			switch(s.length)
			{
				case WHOLE:
				case HALF:
					WinEraseLine(x+2,sy-2*(s.pitch-26),
						x+2,sy-2*(s.pitch-26));
					break;

				case THIRTYSECOND: /* TODO */
				case SIXTEENTH:
					WinDrawLine(x+2+s.stem,
						sy-2*(s.pitch-26)-s.stem*(STEM_LENGTH-3),
						x+2+s.stem+2,
						sy-2*(s.pitch-26)-s.stem*(STEM_LENGTH-3)
							+TAIL_LENGTH*s.stem);
				/* fall through */
				case EIGHTH:
					WinDrawLine(x+2+s.stem,sy-2*(s.pitch-26)-s.stem*STEM_LENGTH,
						x+2+s.stem+2,sy-2*(s.pitch-26)-s.stem*STEM_LENGTH
							+TAIL_LENGTH*s.stem);
				case QUARTER:
					break;
			}

			/* stem: 1 for up, -1 for down */
			if(s.length != WHOLE)
				WinDrawLine(x+2+s.stem,sy-2*(s.pitch-26),
					x+2+s.stem,sy-2*(s.pitch-26)-s.stem*STEM_LENGTH);

			/* dots */
			for(i=0;i<s.dots;i++)
				WinDrawLine(x+5+2*i,sy-2*(s.pitch-26)-1,
					x+5+2*i,sy-2*(s.pitch-26)-1);

			break;

		case REST:

			break;
		
		case BARLINE:
			switch(s.length)
			{
				case B_SINGLE:
					WinDrawLine(x+1, sy, x+1, sy+16);
					break;

				case B_DOUBLE:
					WinDrawLine(x+1, sy, x+1, sy+16);
					WinDrawLine(x+3, sy, x+3, sy+16);
					break;

				case B_RREP:
					WinDrawLine(x+1, sy+6, x+1, sy+6);
					WinDrawLine(x+1, sy+10, x+1, sy+10);
					WinDrawLine(x+3, sy, x+3, sy+16);
					WinDrawLine(x+5, sy, x+5, sy+16);
					break;

				case B_LREP:
					WinDrawLine(x+1, sy, x+1, sy+16);
					WinDrawLine(x+3, sy, x+3, sy+16);
					WinDrawLine(x+5, sy+6, x+5, sy+6);
					WinDrawLine(x+5, sy+10, x+5, sy+10);
					break;

				case B_DREP:
					WinDrawLine(x+1, sy+6, x+1, sy+6);
					WinDrawLine(x+1, sy+10, x+1, sy+10);
					WinDrawLine(x+3, sy, x+3, sy+16);
					WinDrawLine(x+5, sy, x+5, sy+16);
					WinDrawLine(x+7, sy+6, x+7, sy+6);
					WinDrawLine(x+7, sy+10, x+7, sy+10);
					break;

				case B_LFAT:
					WinDrawLine(x+1, sy, x+1, sy+16);
					WinDrawLine(x+2, sy, x+2, sy+16);
					WinDrawLine(x+4, sy, x+4, sy+16);
					break;

				case B_RFAT:
					WinDrawLine(x+1, sy, x+1, sy+16);
					WinDrawLine(x+3, sy, x+3, sy+16);
					WinDrawLine(x+4, sy, x+4, sy+16);
					break;

				case B_FIRST:
					/* bracket */
					WinDrawLine(x-2,sy-2, x-2,sy-7);
					WinDrawLine(x-2,sy-7, x+1,sy-7);
					/* 1 */
					WinDrawLine(x,sy-2, x,sy-5);
					break;

				case B_SECOND:
					/* bracket */
					WinDrawLine(x-2,sy-2, x-2,sy-7);
					WinDrawLine(x-2,sy-7, x+3,sy-7);
					/* 2 */
					WinDrawLine(x,sy-5, x+1,sy-5);
					WinDrawLine(x+2,sy-4, x,sy-2);
					WinDrawLine(x,sy-2, x+2,sy-2);
					break;
			}
			break;

		case CLEF:

			break;

		case TIMESIG:
			StrIToA(tt,s.ts1);
			WinDrawChars(tt,StrLen(tt),x+1,sy-2);
			StrIToA(tt,s.ts2);
			WinDrawChars(tt,StrLen(tt),x+1,sy+8);
			/* TODO: possible not to trash stave lines? */
			/* WinDrawLine(x,sy+8,x+10,sy+8); */
			break;

		case KEYSIG:

			break;

		case TEXTLINE:
			WinDrawChars(ABCtext[s.ti],StrLen(ABCtext[s.ti]),2, y);
			break;

		default:

			break;
	}
}



/* draw the music */
void RedrawMusicForm(void)
{
	/* I've been putting off writing this for ages... */

	UInt screenstave,x,y,i,j,height;

	y=STAVE_OFFSET;
	i=0; /* position in ABCsymbols */

	for(screenstave=0;screenstave<20;screenstave++) /* 20 is a safety net */
	{
		x=0;

		if(ABCsymbols[i].type==TEXTLINE)
		{
			height=10;
		}
		else
		{
			height=STAVE_SEP;
			/* draw a stave at y */
			for(j=0;j<5;j++)
				WinDrawLine(0,(STAVE_SEP/2)-8+y+j*4,
					159,(STAVE_SEP/2)-8+y+j*4);
		}

		while(ABCsymbols[i].stave==screenstave)
		{
			drawsymbol(x,y,ABCsymbols[i]);
			/* we already know this won't take us off the side of the
			 * screen */
			x+=ABCsymbols[i].width;

			i++;
			if(i>=nsyms)
			{
				screenstave=20;
				break;
			}
		}


	y+=height;

	}

	
#if 0
	UInt	i,j,y;

	for(i=0;i<4;i++)
	{
		y=STAVE_OFFSET+i*STAVE_SEP;
		if(ABCdata[i+TopStave][0]!=0)
		{
			if(ABCdata[i+TopStave][0]!=ABCCOMP)
			{
				for(j=0;j<5;j++)
					WinDrawLine(0,y+j*4,
						159,y+j*4);
				/* treble clef */
				WinDrawLine(4,y+17,8,y+19);
				WinDrawLine(8,y+19,10,y+17);
				WinDrawLine(10,y+16,5,y-1);
				WinDrawLine(5,y-1,8,y-4);
				WinDrawLine(8,y-4,10,y-1);
				WinDrawLine(10,y-1,11,y+2);
				WinDrawLine(11,y+2,5,y+9);
				WinDrawLine(5,y+9,4,y+12);
				WinDrawLine(4,y+12,8,y+14);
				WinDrawLine(8,y+14,10,y+10);
				WinDrawLine(10,y+10,8,y+9);
				WinDrawLine(8,y+9,6,y+11);
			}

			/* draw the line of music */
			for(j=0;j<maxThings;j++)
			{
				switch(ABCdata[i+TopStave][j])
				{
					case ABCEND:
						j=maxThings;
						break;

					case ABCBAR:
						WinDrawLine(10*j+17,y,10*j+17,y+16);
						break;

					case ABCDBAR:
						WinDrawLine(10*j+16,y,10*j+16,y+16);
						WinDrawLine(10*j+18,y,10*j+18,y+16);
						break;

					case ABCLREP:
						WinDrawLine(10*j+16,y,10*j+16,y+16);
						WinDrawLine(10*j+18,y,10*j+18,y+16);
						WinDrawLine(10*j+20,y+6,10*j+20,y+6);
						WinDrawLine(10*j+20,y+10,10*j+20,y+10);
						break;

					case ABCRREP:
						WinDrawLine(10*j+20,y,10*j+20,y+16);
						WinDrawLine(10*j+18,y,10*j+18,y+16);
						WinDrawLine(10*j+16,y+6,10*j+16,y+6);
						WinDrawLine(10*j+16,y+10,10*j+16,y+10);
						break;

					case ABCCOMP:
						/*
						if(*ABCcomposer!='\0')
							WinDrawChars(ABCcomposer,StrLen(ABCcomposer),
								2,STAVE_OFFSET+i*STAVE_SEP);
						j=maxThings;
						*/
						break;

					default: /* ordinary note */
						WinDrawLine(10*j+16,y+16+8-ABCdata[i+TopStave][j]*2-3,
							10*j+16,y+16+8-ABCdata[i+TopStave][j]*2-1);
						WinDrawLine(10*j+17,y+16+8-ABCdata[i+TopStave][j]*2-4,
							10*j+17,y+16+8-ABCdata[i+TopStave][j]*2);
						WinDrawLine(10*j+18,y+16+8-ABCdata[i+TopStave][j]*2-3,
							10*j+18,y+16+8-ABCdata[i+TopStave][j]*2-1);
						break;
				}
			}

		}
	}
#endif /* 0 */
}


Boolean MusicFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
	/* Err err; */
	GoToParamsPtr	gotoPtr;
	LocalID			DBID;
	UInt			card = 0;
	DmSearchStateType ss;
    
    CALLBACK_PROLOGUE

    switch (e->eType)
	{
		case frmOpenEvent:
		frm = FrmGetActiveForm();

		/* set up form... */
		LoadMusicData(frm);

		/* draw the UI parts of the form */
		FrmDrawForm(frm);

		/* and the rest */
		RedrawMusicForm();

		handled = true;
		break;

		case menuEvent:
		MenuEraseStatus(NULL);

		switch(e->data.menu.itemID) {
			case MusicEditItem:
				/* the following is based on code from
				 * http://www.palm.com/devzone/faqs/HHOS.html#a132
				 * ('cos I couldn't figure it out myself!)
				 */
				gotoPtr=MemPtrNew(sizeof(GoToParamsType));
				if(gotoPtr)
				{
					MemPtrSetOwner(gotoPtr,0);
					DmGetNextDatabaseByTypeCreator(true,&ss,'DATA',
						sysFileCMemo, true, &card, &DBID);
					gotoPtr->searchStrLen=0;
					gotoPtr->dbCardNo=0;
					gotoPtr->dbID=DBID;
					gotoPtr->recordNum=CurrentRecord;
					gotoPtr->matchPos=0;
					gotoPtr->matchFieldNum=0;
					gotoPtr->matchCustom=0;
					DmGetNextDatabaseByTypeCreator(true,&ss,
						sysFileTApplication,sysFileCMemo,true,&card,&DBID);
					SysUIAppSwitch(0,DBID,sysAppLaunchCmdGoTo,(Ptr)gotoPtr);
				}
				else
				{
					Alert("Couldn't allocate memory (gotoPtr)"," "," ");
				}
				handled=true;
			break;

			case MusicCloseItem:
				FrmGotoForm(MainForm);
				handled=true;
			break;
			}

			handled = true;
		break;

		case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID)
		{
		}
		break;

		default:
			break;
    }

    CALLBACK_EPILOGUE

    return handled;
}


void MainFormScroll(Int lines, Boolean page) /* +ve for down, -ve for up */
{
	TablePtr table;
    FormPtr frm;
	Word	recordNum,r1;
	Int val;
	Int		rows;

	/* please, someone tell me there's an easier way... */
	frm = FrmGetActiveForm();
	table=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainTable));
	recordNum=TopVisibleRecord;
	rows=TblGetNumberOfRows(table);

	val=lines;

	if(page)
	{
		val*=(rows-1);
	}

	if(val>0) /* down */
	{
		if(DmSeekRecordInCategory(MemoPadDB,&recordNum,val,
			dmSeekForward,ABCcategory)!=0)
		{
			recordNum=dmMaxRecordIndex;
			DmSeekRecordInCategory(MemoPadDB,&recordNum,rows-1,
				dmSeekBackward,ABCcategory);
		}
		else /* it worked, but are we too near the bottom?? */
		{
			r1=recordNum;
			if(DmSeekRecordInCategory(MemoPadDB,&r1,rows-1,
				dmSeekForward,ABCcategory))
			{
			recordNum=dmMaxRecordIndex;
			DmSeekRecordInCategory(MemoPadDB,&recordNum,rows-1,
				dmSeekBackward,ABCcategory);
			}
		}
	}
	else if(val<0) /* up */
	{
		if(DmSeekRecordInCategory(MemoPadDB,&recordNum,abs(val),
			dmSeekBackward,ABCcategory))
		{
			recordNum=0;
			DmSeekRecordInCategory(MemoPadDB,&recordNum,0,
				dmSeekForward,ABCcategory);
		}
	}
	else
	{
		Alert("MainFormScroll with val=0"," "," ");
	}


	/*
	DmSeekRecordInCategory(MemoPadDB,&recordNum,abs(val),
		(val>0) ? dmSeekForward : dmSeekBackward ,ABCcategory);
	*/

	if(TopVisibleRecord!=recordNum) /* only redraw if necessary */
	{
		TopVisibleRecord=recordNum;
		MainViewLoadTable(frm,TopVisibleRecord);
		TblRedrawTable(table);
	}
}


Boolean MainFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    FormPtr aboutFrm;
	TablePtr table;
    
    CALLBACK_PROLOGUE

    switch (e->eType)
	{
		case frmOpenEvent:
			frm = FrmGetActiveForm();
			/* load in the records for the table */
			MainViewLoadRecords(frm);
			FrmDrawForm(frm);

			/* enable the scrollbar */
			table=FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainTable));
			TblHasScrollBar(table,true);

			handled = true;
			break;

		case menuEvent:
			MenuEraseStatus(NULL);
			switch(e->data.menu.itemID)
			{
				case MainHelpItem:
				FrmHelp(HelpString);
				handled=true;
				break;

				case MainAboutItem:
				aboutFrm=FrmInitForm(AboutForm);
				FrmDoDialog(aboutFrm);
				FrmDeleteForm(aboutFrm);
				handled=true;
				break;
			}
			handled = true;
			break;

		case ctlSelectEvent:
			switch(e->data.ctlSelect.controlID) {
			}
			break;

		case keyDownEvent:
			if(e->data.keyDown.chr==pageUpChr)
			{
				MainFormScroll(-1,true);
				//handled=true;
			}
			else if(e->data.keyDown.chr==pageDownChr)
			{
				MainFormScroll(1,true);
				//handled=true;
			}
			break;

		case sclRepeatEvent:
			MainFormScroll(e->data.sclRepeat.newValue -
				e->data.sclRepeat.value,false);
			break;

		case tblSelectEvent:
			CurrentRecord=TblGetItemInt(e->data.tblSelect.pTable,
				e->data.tblSelect.row, e->data.tblSelect.column);
			
			/* display it */
			FrmGotoForm(MusicForm);

			handled=true;
			break;

		default:
			break;
    }

    CALLBACK_EPILOGUE

    return handled;
}

Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Word    formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case MainForm:
	    FrmSetEventHandler(frm, MainFormHandleEvent);
	    break;
	case MusicForm:
		FrmSetEventHandler(frm, MusicFormHandleEvent);
		break;
	}
	handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
Word StartApplication(void)
{
	UInt mode;
	SystemPreferencesType sysPrefs;

    FrmGotoForm(MainForm);

	/* show secret records? */
	PrefGetPreferences(&sysPrefs);
	if(sysPrefs.hideSecretRecords)
		mode=dmModeReadOnly;
	else
		mode=dmModeReadOnly | dmModeShowSecret;

	MemoPadDB=DmOpenDatabaseByTypeCreator('DATA','memo',mode);
	if(!MemoPadDB)
	{
		Alert("Couldn't open MemoPad database"," "," ");
		return(1);
	}

	ABCcategory=CategoryFind(MemoPadDB,"ABC");
	if(ABCcategory==255)
	{
		Alert("No ABC category"," ","(please create it in MemoPad)");
		return(1);
	}

	/* "dumb" startup */
	TopVisibleRecord=1;
	CurrentRecord=noRecordSelected;
	MemosInCategory=memosInCategoryUnknown;

    return 0;
}

/* Save preferences, close forms, close app database */
void StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
	DmCloseDatabase(MemoPadDB);
}

/* The main event loop */
void EventLoop(void)
{
    Word err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Word err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

	err = StartApplication();
	if (err) return err;

	EventLoop();
	StopApplication();

    } else {
	return sysErrParamErr;
    }

    return 0;
}

/* vi:ts=4:sw=4:si:nocindent
 */
