/* parse.c */

/*

ABC Viewer, a program to display abc format music files on the Palm Pilot.

    Copyright (C) 1998,1999  Dave Holland  dave@zenda.demon.co.uk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <Pilot.h>
#include "callback.h"

#include "abcview.h"
#include "parse.h"

#include "abcviewRsc.h"


/* global variables */

extern DmOpenRef	MemoPadDB;
extern Word			ABCcategory;

/* the mystical "a" pointer */
extern CharPtr		a;

extern UInt			defaultLength;

extern UInt	pplet,qplet,rplet;

/* the main symbol array */
extern struct SYMBOL	ABCsymbols[maxSymbols];
extern UInt			nsyms;

/* the text array */
extern char			ABCtext[maxText][maxTextLen];
extern Int			ntext;


extern UInt			CurrentRecord;
extern Int			TopVisibleRecord;
extern Word			MemosInCategory;
extern UInt			TopStave;
extern Word			lastRecordNum;


UInt	numtitle;
Int		carryover;

/* copy '\n' terminated line */
void CopyLine(CharPtr d, CharPtr s)
{
	CharPtr i;

	for(i=s; (*i != '\n') && (*i != '\0'); i++, d++)
	{
		*d=*i;
	}
	*d='\0';
}

/* add_text - add a text line into the ABCtext array */
int add_text(CharPtr t)
{
	CopyLine(&ABCtext[ntext][0],t);
	ntext++;
	return(ntext-1);
}

/* parse an unsigned integer at "a" */
UInt parse_uint()
{
	UInt number=0;

	if(!StrChr("0123456789",*a)) return(0);

	while(StrChr("0123456789",*a))
	{
		number*=10;
		number+=((*a)-'0');
		a++;
	}
	return(number);
}

/* parse_length - parse note/rest length specifier */
int parse_length()
{
	CharPtr b;
	int l,f;

	b=a;
	l=defaultLength; /* default length for this meter */

	if(StrChr("0123456789",*a))
	{
		f=parse_uint();
		if(f==0) f=1;
		l=l*f;
	}

	if(*a=='/')
	{
		while(*a=='/')
		{
			a++;
			if(StrChr("0123456789",*a))
				f=parse_uint();
			else
				f=2;
			if(l%f) /* TODO: flag bad length divisor */
			{
				Alert("bad length divisor at"," ",b);
				return(l);
			}
			l=l/f;
		}

	}

	return(l);
}

/* numeric_pitch - stolen from abc2ps, in turn from abc2mtex */
/* middle C => 16 */
int numeric_pitch(char n)
{
	switch(n)
	{
		case 'z':
			return(14);
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'G':
			return(n-'C'+16);
		case 'A':
		case 'B':
			return(n-'A'+21);
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 'g':
			return(n-'c'+23);
		case 'a':
		case 'b':
			return(n-'a'+28);
		default:
			return(0);
	}
}

/* is the line at a an "info" line? */
Boolean is_info(CharPtr a)
{
	if((*a != '\0') && (*a != '|') && (*(a+1)==':'))
		return(true);
	return(false);
}


/* add a symbol to the array */
void add_symbol(struct SYMBOL s)
{
	nsyms++;
	if(nsyms==maxSymbols)
	{
		Alert("Too many symbols!"," "," ");
		return;
	}
	ABCsymbols[nsyms-1]=s;
}

/* calculate the note widths of all the symbols in the array */
void calculate_widths()
{
	Int i,w;

	for(i=0;i<nsyms;i++)
	{
		w=0;
		switch(ABCsymbols[i].type)
		{
			case NOTE:
				/* TODO: fix this for slurs etc */
				w=5; /* 3 for head, 2 for spacing */
				w+=2*ABCsymbols[i].dots;
				if((ABCsymbols[i].length<=EIGHTH) &&
					(ABCsymbols[i].stem==STEM_UP)) w++;
				switch(ABCsymbols[i].acc)
				{
					case A_SH:
						w+=6;
						break;

					case A_NT:
					case A_FL:
						w+=4;
						break;
				}
				break;
			case REST:
				w=5; /* TODO: fix this */
				break;
			case BARLINE:
				switch(ABCsymbols[i].length)
				{
					case B_SINGLE:
						w=3;
						break;
					case B_DOUBLE:
						w=5;
						break;
					case B_LREP:
					case B_RREP:
						w=7;
						break;
					case B_LFAT:
					case B_RFAT:
						w=6;
						break;
					case B_DREP:
						w=9;
						break;
					default: /* TODO: currently catches 1st/2nd endings */
						w=3;
						break;
				}
				break;
			case CLEF:
				w=10; /* TODO: fix this? */
				break;
			case TIMESIG:
				if((ABCsymbols[i].ts1>9) || (ABCsymbols[i].ts2>9))
					w=11;
				else
					w=6;
				break;
			case KEYSIG:
				w=10; /* TODO: fix this */
				break;
			case TEXTLINE:
				w=9999;
				break;
			default:
				Alert("Unknown symbol type in calculate_widths"," "," ");
				break;
		}
		ABCsymbols[i].width=w;

	}

}


/* parse text line for default note length */
UInt parse_defaultlen(CharPtr b)
{
	int l1=0,l2=1;
	int d,dlen;
	CharPtr c;

	/* sscanf(b,"%d/%d",&l1,&l2); */
	
	/* DIY sscanf() */
	c=StrChr(b,'/');
	if(c)
	{
		l1=StrAToI(b);
		l2=StrAToI(c+1);
	}
	else
	{
		Alert("Bad length:"," ",b);
		return(defaultLength);
	}
	
	if(l1==0)
		return(defaultLength); /* empty string => no change */

	d=BASE/l2;
	if(d*l2 != BASE )
	{
		Alert("Length ", b, " incompatible with BASE: using 1/8");
		dlen=BASE/8;
	}
	else
	{
		dlen=d*l1;
	}

	return(dlen);

}



/* parse a time sig string */
void parse_timesig(CharPtr b, struct SYMBOL * sp)
{
	CharPtr c;
	
	if((*b == '\0') || (*b == '\n'))
	{
		Alert("Empty meter string"," "," ");
		return;
	}

	/* special cases */
	/* TODO: tag these so they display as "C" not "4/4" etc */
	if(*b == 'C')
	{
		if(*(b+1) == '|')
		{
			sp->ts1=2;
			sp->ts2=2;
			return;
		}
		else
		{
			sp->ts1=4;
			sp->ts2=4;
			return;
		}
	}

	c=StrChr(b,'/');
	if(c)
	{
		sp->ts1=StrAToI(b);
		sp->ts2=StrAToI(c+1);
	}
	else
	{
		Alert("Bad meter (missing '/'):"," ",b);
	}
}



/* work through the symbol widths and split into staves */
void allocate_staves(void)
{
	Int i,s=0,sofar=0;

	for(i=0;i<nsyms;i++)
	{
		sofar+=ABCsymbols[i].width;
		if(sofar>159)
		{
			sofar=ABCsymbols[i].width;
			s++;
		}
		ABCsymbols[i].stave=s;
	}
	/* TODO: split bars in sensible places (like at barlines) */
	/* TODO: justification of symbols on stave */
}


void parse_info_line(FormPtr frm)
{
	char	tt[maxTextLen];
	struct SYMBOL s;

			switch(*a)
			{
				case 'A': /* area */
					break;
				case 'B': /* book */
					break;
				case 'C': /* composer */
					s.ti=add_text(a+2);
					s.type=TEXTLINE;
					add_symbol(s);
					break;
				case 'D': /* discography */
					break;
				case 'G': /* group */
					break;
				case 'H': /* history */
					/* TODO: handle multi-line */
					break;
				case 'I': /* info */
					break;
				case 'K': /* key */
					break;
				case 'L': /* default note length */
					defaultLength=parse_defaultlen(a+2);
					break;
				case 'M': /* meter */
					s.type=TIMESIG;
					s.ts2=0;
					parse_timesig(a+2,&s);
					add_symbol(s);
					if(s.ts2 != 0)
					{
						if((float)s.ts1/(float)s.ts2 < 0.75)
							defaultLength=BASE/16;
						else
							defaultLength=BASE/8;
						/*
						StrIToA(tt,defaultLength);
						Alert("defaultLength","=",tt);
						*/
					}
					break;
				case 'N': /* textual notes */
					s.ti=add_text(a+2);
					s.type=TEXTLINE;
					add_symbol(s);
					break;
				case 'O': /* origin */
					break;
				case 'P': /* parts */
					break;
				case 'Q': /* tempo */
					break;
				case 'R': /* rhythm */
					break;
				case 'S': /* source */
					break;
				case 'T': /* title */
					numtitle++;
					if(numtitle==1)
					{
						CopyLine(tt,a+2);
						FrmCopyTitle(frm,tt);
					}
					else
					{
						s.ti=add_text(a+2);
						s.type=TEXTLINE;
						add_symbol(s);
					}
					break;
				case 'W': /* words */
					break;
				case 'X': /* cross-ref number */
					break;
				case 'Z': /* transcription note */
					break;
				default: /* unknown field type */
					Alert("Unknown info field type"," ",a);
					break;
			}		

			while((*a != '\0') && (*a != '\n')) a++;
			if(*a == '\n') a++;

}


/* set len and dots for a note */
void do_note(int n)
{
	int len,base,dots;
	char	tt[maxTextLen];

	len=ABCsymbols[n].length;

	/* work out what the basic note length is,
	 * discounting dots */
	base=WHOLE;
	if(len>=WHOLE)				base=WHOLE;
	else if(len>=HALF)			base=HALF;
	else if(len>=QUARTER)		base=QUARTER;
	else if(len>=EIGHTH)		base=EIGHTH;
	else if(len>=SIXTEENTH)		base=SIXTEENTH;
	else if(len>=THIRTYSECOND)	base=THIRTYSECOND;
	else
	{
		StrIToA(tt,len);
		Alert("Eeek, bad length (",tt,")");
	}

	dots=0;
	if(len==base)			dots=0;
	else if(2*len==3*base)	dots=1;
	else if(4*len==7*base)	dots=2;
	else if(8*len==15*base)	dots=3;
	else Alert("Freaky note length"," "," ");

	ABCsymbols[n].dots=dots;
	ABCsymbols[n].length=base;
}


/* modify a note's length for ">" and "<" signs. n=which note, num=how many
 * ">"'s, sign= 1 for longer, -1 for shorter.
 * Cribbed shamelessly from abc2ps's double_note routine */

void double_note(int n,int num,int sign)
{
	int	shift,len,j;

	shift=0;
	len=ABCsymbols[n].length;
	for(j=0;j<num;j++)
	{
		len=len/2;
		shift -= sign*len;
		ABCsymbols[n].length += sign*len;
	}
	carryover += shift;
	do_note(n);
}



/* load in the tune data */
void LoadMusicData(FormPtr frm)
{
	VoidHand	recHandle;
	CharPtr recText;
	struct SYMBOL s;
	Int		acc,len,num;
	Int		last_note=-1;
	//char	tt[maxTextLen];

	/* Lock down the record from the database */
	recHandle=DmGetRecord(MemoPadDB,CurrentRecord);
	recText=MemHandleLock(recHandle);

	/* Parsing strategy:
	 * 1. from *recText, parse the abc into symbols which are stored in
	 *    ABCsymbols
	 * 2. calculate the symbol widths and split into lines
	 */

	/* TODO: completely forget old tune data/params */
	nsyms=0;
	ntext=0;
	numtitle=0;
	defaultLength=QUARTER; /* TODO !!! */
	carryover=0;
	pplet=qplet=rplet=0;

	/* parse it */
	a=recText;
	while(*a)
	{
		/* handle info fields */
		if(is_info(a))
		{
			parse_info_line(frm);
		}
		else if (*a=='%') /* comment line */
		{
			while((*a != '\0') && (*a != '\n')) a++;
			if(*a == '\n') a++;
		}
		else
		{
			/* parse this line as characters */
			while((*a != '\0') && (*a != '\n'))
			{
				if (*a=='"') /* guitar chord, etc */
				{
					a++;
					while((*a != '\0') && (*a != '\n') && (*a != '"')) a++;
					if(*a != '\0') a++;
					/* TODO: EOT in gchord */
					/* if(*a == '\n') a++; */
				}
				else if (*a==' ') a++; /* TODO: groups of notes */
				else if (*a=='\\') a++; /* alignment flag -- ignored */
				else if (*a=='*') a++; /* alignment flag -- ignored */
				else if (*a=='~') a++; /* decoration -- ignored */
				else if (*a=='>')
				{
					num=1;
					a++;
					while(*a == '>') { num++; a++; }
					if(last_note<0)
						Alert("No note before > at"," ",a);
					else
						double_note(last_note,num,1);
				}
				else if (*a=='<')
				{
					num=1;
					a++;
					while(*a == '<') { num++; a++; }
					if(last_note<0)
						Alert("No note before < at"," ",a);
					else
						double_note(last_note,num,-1);
				}
				else if (*a=='-') /* tie */
				{
					a++;
					/* TODO: mark last note as start of slur */
				}
				else if (*a=='\(') /* n-plet */
				/* (p:q:r means "put p notes into the time of q for the next r
				 * notes"
				 * abc2ps parse.h line 2098 */
				{
					a++;
					pplet=parse_uint();
					if(pplet>0)
					{
						qplet=0;
						rplet=pplet;
						if(*a==':')
						{
							a++;
							qplet=parse_uint();
							if(*a==':')
							{
								a++;
								rplet=parse_uint();
							}
						}
					}
					// StrPrintF(tt,"%d %d %d",pplet,qplet,rplet);
					// Alert("Parsed n-plet:"," ",tt);
				}
				else if (*a==':')
				{
					s.type=BARLINE;
					a++;
					switch(*a)
					{
						case '|':
							a++;
							if(*a == '2')
							{ /* 2nd time. parse as ":|" followed by "2" */
								a++;
								s.length=B_RREP;
								add_symbol(s);
								s.length=B_SECOND;
								add_symbol(s);
							}
							else
							{
								s.length=B_RREP;
								add_symbol(s);
							}
							break;
						case ':':
							a++;
							s.length=B_DREP;
							add_symbol(s);
							break;
					}
					
				}
				else if (*a=='|')
				{
					s.type=BARLINE;
					a++;
					switch(*a)
					{
						case '|':
							a++;
							s.length=B_DOUBLE;
							add_symbol(s);
							break;
						case ':':
							a++;
							s.length=B_LREP;
							add_symbol(s);
							break;
						case ']':
							a++;
							s.length=B_RFAT;
							add_symbol(s);
							break;
						case '1': /* 1st time. parse as "|" followed by "1" */
							a++;
							s.length=B_SINGLE;
							add_symbol(s);
							s.length=B_FIRST;
							add_symbol(s);
							break;
						default:
							s.length=B_SINGLE;
							add_symbol(s);
							break;
					}
				}
				else if (*a=='[')
				{
					s.type=BARLINE;
					a++;
					switch(*a)
					{
						case '|':
							a++;
							s.length=B_LFAT;
							add_symbol(s);
							break;
						case '1':
							a++;
							s.length=B_FIRST;
							add_symbol(s);
							break;
						case '2':
							a++;
							s.length=B_SECOND;
							add_symbol(s);
							break;
					}
				}
				else
				{
					/* "real" music */ /* abc2ps parse.h line 2010 */
					if(StrChr("XxZzCDEFGABcdefgab^=_",*a))
					{
						if(StrChr("XxZz",*a))
						{
							a++;
							s.type=REST;
							s.length=parse_length();
							add_symbol(s);
							/* TODO: dotted rests */
						}
						else
						{
							s.type=NOTE;
							/* parse for pitch, then length */
							acc=0;
							switch(*a)
							{
								case '=':
									acc=A_NT;
									break;
								case '^':
									acc=A_SH;
									if(*(a+1)=='^') { acc=A_DS; a++; }
									break;
								case '_':
									acc=A_FL;
									if(*(a+1)=='_') { acc=A_DF; a++; }
									break;
							}

							s.acc=acc;

							if(acc) a++;

							/* if(!StrChr("CDEFGABcdefgab",*a))
								return(0); */

							s.pitch=numeric_pitch(*a);
							a++;

							while(*a == '\'')
							{
								s.pitch+=7;
								a++;
							}

							while(*a == ',')
							{
								s.pitch-=7;
								a++;
							}

							len=parse_length();

							if(len+carryover<=0)
							{
								Alert("> leads to zero or negative note length",
									" at ",a);
							}
							else
							{
								len += carryover;
							}

							carryover=0;

							s.length=len;

							/* set stem direction quite naively */
							if(s.pitch==22)
							{
								if((last_note>=0) &&
										(ABCsymbols[last_note].pitch>=21) &&
										(ABCsymbols[last_note].pitch<=23))
									s.stem=ABCsymbols[last_note].stem;
								else
									s.stem=STEM_UP;
							}
							else if(s.pitch>22)
							{
								s.stem=STEM_DOWN;
							}
							else
							{
								s.stem=STEM_UP;
							}

							add_symbol(s);
							last_note=nsyms-1;
							do_note(last_note);
						}

					}
					else
					{ /* so what was it?? */
					Alert("Unknown symbol while parsing"," ",a);
					a++;
					}

				}

			}
			/* if(*a != '\0') a++; */
			if(*a == '\n') a++;

		}
	}

	calculate_widths();
	allocate_staves();

	/* unlock and release */
	MemHandleUnlock(recHandle);
	DmReleaseRecord(MemoPadDB,CurrentRecord,false); /* not dirty */

}


/* vi:ts=4:sw=4:si:nocindent
 */
